// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "display/shader/ShaderData.hpp"
#include "assert/assertgl.hpp"

using namespace Dawn;

ShaderData::ShaderData(
  const std::shared_ptr<ShaderStructure> &structure
) : IShaderData(structure) {
  glGenBuffers(1, &this->buffer);
  assertNoGLError();
}

void ShaderData::upload() {
  if(!this->dirty) return;

  glBindBuffer(GL_UNIFORM_BUFFER, this->buffer);
  assertNoGLError();
  glBindBufferBase(GL_UNIFORM_BUFFER, 0, this->buffer);
  assertNoGLError();
  glBufferData(GL_UNIFORM_BUFFER, this->structure->size, this->data, GL_STATIC_DRAW);
  assertNoGLError();
  
  this->dirty = false;
}

void ShaderData::bind(const GLuint index) {
  glBindBufferBase(GL_UNIFORM_BUFFER, index, this->buffer);
  assertNoGLError();
}

ShaderData::~ShaderData() {
  if(this->buffer != -1) {
    glDeleteBuffers(1, &this->buffer);
    assertNoGLError();
    this->buffer = -1;
  }
}